@@ -1,14 +1,18 @@
-'.chunk' <- function(nr,size=96,name="")
-{
+'.chunk' <- function(nr,size=96,selected=NULL,name="") {
+   if (indexed <- !is.null(selected))
+      nr <- length(selected)
    nl <- ceiling(nr/size)
    res <- vector("list",nl)
    i1 <- 1
-   for (i in 1:nr)
-   {
+  # if (is.null(selected))
+  #    seqi <- seq(nr)
+  # else
+  #    seqi <- selected
+   for (i in seq(nr)) {
       i2 <- i1+size-1
       if (i2>nr)
          i2 <- nr
-      res[[i]] <- i1:i2
+      res[[i]] <- if (indexed) selected[i1:i2] else i1:i2
       i1 <- i2+1
       if (i1>nr)
          break
@@ -17,8 +21,7 @@
       names(res) <- sprintf(sprintf("%s%%0%dd",name,nchar(length(1:i))),1:i)
    res
 }
-'chunk_line' <- function(obj,mem=100,mul=1)
-{
+'chunk_line' <- function(obj,mem=100,mul=1,selected=NULL) {
    if (!is.ursa(obj))
       return(NULL)
    y <- obj$con
@@ -33,10 +36,9 @@
       nc <- y$samples
    }
    a <- as.integer(ceiling(1e6*mem*mul/(8*nc*nb)))
-   return(.chunk(nr,a,"line"))
+   return(.chunk(nr,a,selected,"line"))
 }
-'chunk_band' <- function(obj,mem=100,mul=1)
-{
+'chunk_band' <- function(obj,mem=100,mul=1,selected=NULL) {
    if (!is.ursa(obj))
       return(NULL)
    y <- obj$con
@@ -51,7 +53,8 @@
       nc <- y$samples
    }
    a <- as.integer(ceiling(1e6*mem*mul/(8*nc*nr)))
-   return(.chunk(nb,a,"band"))
+  # print(data.frame(nb=nb,a=a,row.names="band"))
+   return(.chunk(nb,a,selected,"band"))
 }
 'chunk_expand' <- function(ind,size=3) {
    nr <- session_grid()$rows
